import logging as lg
import pandas as pd
from pandas_profiling import ProfileReport
from sklearn.preprocessing import StandardScaler
import ydata_profiling
from statsmodels.stats.outliers_influence import variance_inflation_factor



class data_preprocessing:
  """
  This is the class for basic data preprocessing it includes several basic method for mean imputation of the null values in the dataset,
  finding multicollinearity among the datasets,
  standardization of the selected of the selected columns in the dataset and 
  pandas profiling of the dataset
  Attributes:
        df: pandas dataframe
        m:  index of the column in the dataset
        n: index of the column in the dataset
  
  """ 
  lg.basicConfig(filename= "D:\\FSDS\\MAchine_Learning\\data_preprocessing.log", level = lg.INFO, format= '%(asctime)s %(message)s')
  lg.info("This is the starting of the class")
  try:
     def __init__(self,df,m,n):
        """
        This is the constructor for data preprocessing class.
        Parameters:
                    df: pandas dataframe
                    m:  index of the column in the dataset
                    n: index of the column in the dataset
        
        
        """
        lg.info("The is the is constructor method of the class")
        lg.info(str("value of df=")+str(df)+str("value of m=")+str(m)+str(",")+str("value of n=")+str(n))
        self.df = df
        self.m = m
        self.n = n
  except Exception as e:
        print("You can check the log for more info")
        lg.error("The error has ocurred")
        lg.exception(str(e))
  try:     
      def mean_imputation(self):
        """
        This is the method for filling the null values in the row with the mean 
        of the the columns in the dataset.
        
        Returns:
                It returns the pandas dataframe after applying the function on the 
                selected coulmns of the dataset. 
        
        """    
        lg.info("The is the is mean imputation method of the class")
        d = {}
        x =pd.DataFrame(self.df.isnull().sum())
        x.reset_index(level=0, inplace=True)
        for i in range(len(x)):
          #print(x.iloc[i,0],x.iloc[i,1])
          d[x.iloc[i,0]] = x.iloc[i,1]
        for j,k in d.items():
          if k != 0:
            self.df[j] = self.df[j].fillna(self.df[j].mean())
        return self.df
  except Exception as e:
        print("You can check the log for more info")
        lg.error("The error has ocurred")
        lg.exception(str(e))
  try:
      def standard_scaling(self):
        """
        This is the method for standardization of the columns in the dataset,
        which are selected for training.

        Returns:
                It returns the pandas dataframe after applying the standardization on the 
                selected coulmns of the dataset. 
        
        """    
        lg.info("The is the is method which will do the standardization of the dataset")    
        self.x = self.df.iloc[:,self.m:self.n]
        self.scaler = StandardScaler()
        arr = self.scaler.fit_transform(self.x)
        return arr
  except Exception as e:
    print("You can check the log for more info")
    lg.error("The error has ocurred")
    lg.exception(str(e))
    
  try:
      def check_multicollinearity(self,x):
        """
        This is the method for applying variation inflation factor of the columns in the dataset,
        and to check for the multicollinerity in the feature columns.
        Threshold value =10 if it is more than that,that feature column to be dropped, from the 
        training dataset.

        Returns:
                It returns the pandas dataframe after applying the variation inflation factor on the 
                selected columns of the dataset and with their respective vif values. 
        
        """      
        lg.info("The is the method which will check the variation inflation factor of the dataset")    
        self.x = x
        arr = self.scaler.fit_transform(x)
        vif_df=pd.DataFrame()
        vif_df["vif"] = [variance_inflation_factor (arr,i) for i in range(arr.shape[1])]
        vif_df["features"] = x.columns
        return vif_df
  except Exception as e:
    print("You can check the log for more info")
    lg.error("The error has ocurred")
    lg.exception(str(e))
    
  try:  
      def pandas_profiling(self):
        """
        This method will return the pandas profile for all the features present in the datasets.
        
        """ 
        lg.info("The is the is method which will do the pandas profiling of the dataset")    
        pf= ProfileReport(self.df)
        return pf
  except Exception as e:
    print("You can check the log for more info")
    lg.error("The error has ocurred")
    lg.exception(str(e))